<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class Restriction extends \Zeedhi\Framework\Controller\Simple {

    protected $restrictionService;
    protected $snackMessage;
    protected $util;

    public function __construct(\Service\Restriction $restrictionService, \Util\SnackMessage $snackMessage, \Util\Util $util){
        $this->restrictionService = $restrictionService;
        $this->snackMessage = $snackMessage;
        $this->util = $util;
    }

    //********************//
    // VALUE RESTRICTIONS //
    //********************//

    /* Gets the families that have restrictions associated to them. */
    public function getValueRestrictions(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];

            $valueRestrictions = $this->restrictionService->getValueRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            foreach($valueRestrictions as &$family){
                $family['VRCONSDIARAL'] = number_format((float)$family['VRCONSDIARAL'], 2, ',', '');
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValueRestrictionsRepository', $valueRestrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Gets a list of families that don't yet have any restrictions associated to them. */
    public function getUnrestrictedFamilies(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];

            $families = $this->restrictionService->getUnrestrictedFamilies($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('UnrestrictedFamiliesRepository', $families));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Adds a family restriction for the user and returns an updated list of the restrictions. */
    public function addValueRestriction(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];
            $CDFAMILISALD = $params[3]['value'];
            $VRCONSDIARAL = $params[4]['value'];

            $valueRestriction = $this->restrictionService->getValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);
            if (sizeof($valueRestriction) == 0){
                $this->restrictionService->addValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRCONSDIARAL);
            }
            else {
                $this->restrictionService->updateValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRCONSDIARAL);
            }

            $valueRestrictions = $this->restrictionService->getValueRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            foreach($valueRestrictions as &$family){
                $family['VRCONSDIARAL'] = number_format((float)$family['VRCONSDIARAL'], 2, ',', '');
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValueRestrictionsRepository', $valueRestrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Updates the family restriction for the user and returns an updated list of the restrictions. */
    public function updateValueRestriction(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];
            $CDFAMILISALD = $params[3]['value'];
            $VRCONSDIARAL = $params[4]['value'];

            $this->restrictionService->updateValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $VRCONSDIARAL);

            $valueRestrictions = $this->restrictionService->getValueRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            foreach($valueRestrictions as &$family){
                $family['VRCONSDIARAL'] = number_format((float)$family['VRCONSDIARAL'], 2, ',', '');
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValueRestrictionsRepository', $valueRestrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Removes a family restriction. Restrictions that have products associated to them will not be deleted. */
    public function deleteValueRestriction(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];
            $CDFAMILISALD = $params[3]['value'];

            /* Validate if there are any restricted products in the selected family. */
            $products = $this->restrictionService->getRestrictedProducts($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);
            if (empty($products)){
            /* No restricted products found in the family. */
                $this->restrictionService->deleteValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);
            } else {
            /* Restricted products were found, therefore we can't delete the value restriction. So we will zero! */
                $this->restrictionService->updateValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, null);
            }

            $valueRestrictions = $this->restrictionService->getValueRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            foreach($valueRestrictions as &$family){
                $family['VRCONSDIARAL'] = number_format((float)$family['VRCONSDIARAL'], 2, ',', '');
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ValueRestrictionsRepository', $valueRestrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    //**********************//
    // PRODUCT RESTRICTIONS //
    //**********************//

    /* Gets a list of all the families associated to a consumer and any restricted products associated with each family. */
    public function getProductRestrictions(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];

            $restriction = $this->restrictionService->getProductRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);

            foreach ($restriction as &$prodRestriction){
                $prodRestriction['cdWeekDays'] = array();
                if($prodRestriction['IDBLOQDIARAL'] !== null){
                    for($i = 1; $i <= 7; $i++){
                        if($prodRestriction['IDBLOQDIARAL'][$i-1] === 'S'){
                            array_push($prodRestriction['cdWeekDays'], "$i");
                        }
                    }
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductRestrictionsRepository', $restriction));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Gets a list of all the families the consumer can add restrictions to.*/
    public function getConsumerFamilies(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];

            $families = $this->restrictionService->getConsumerFamilies($CDFILIAL);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ConsumerFamiliesRepository', $families));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Gets a list of products associated with a family. */
    public function getFamilyProducts(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDFAMILISALD = $params[1]['value'];
            $CDCLIENTE    = $params[2]['value'];
            $CDCONSUMIDOR = $params[3]['value'];

            $products = $this->restrictionService->getFamilyProducts($CDFILIAL, $CDFAMILISALD, $CDCLIENTE, $CDCONSUMIDOR);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('FamilyProductsRepository', $products));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Adds a product restriction. If the family associated with the product doesn't have a restriction, a value restriction is then added for that family. */
    public function addProductRestriction(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];
            $CDFAMILISALD = $params[3]['value'];
            $CDPRODUTO    = $params[4]['value'];
            $QTCONSDIARAL = $params[5]['value'];
            $cdWeekDays   = $params[6]['value'];

            /* Checks if the family associated with the product already has a restriction. */
            $valueRestriction = $this->restrictionService->getValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD);
            if (sizeof($valueRestriction) == 0){
                $this->restrictionService->addValueRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, null);
            }

            foreach ($CDPRODUTO as $productCode){
                if(count($cdWeekDays) === 7){
                    $this->restrictionService->addProductRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $productCode, $QTCONSDIARAL, 'SSSSSSS');
                } else{
                    $IDBLOQDIARAL = '';

                    // Domingo
                    if(in_array('1', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Segunda
                    if(in_array('2', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Terça
                    if(in_array('3', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Quarta
                    if(in_array('4', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Quinta
                    if(in_array('5', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Sexta
                    if(in_array('6', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    // Sabado
                    if(in_array('7', $cdWeekDays)){
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                    } else{
                        $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                    }

                    $this->restrictionService->addProductRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $productCode, $QTCONSDIARAL, $IDBLOQDIARAL);
                }
            }

            $productRestrictions = $this->restrictionService->getProductRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);

            foreach ($productRestrictions as &$prodRestriction){
                $prodRestriction['cdWeekDays'] = array();
                if($prodRestriction['IDBLOQDIARAL'] !== null){
                    for($i = 1; $i <= 7; $i++){
                        if($prodRestriction['IDBLOQDIARAL'][$i-1] === 'S'){
                            array_push($prodRestriction['cdWeekDays'], "$i");
                        }
                    }
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductRestrictionsRepository', $productRestrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Updates a product restriction. */
    public function updateProductRestriction(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $QTCONSDIARAL = $params[0]['value'];
            $CDFILIAL     = $params[1]['value'];
            $CDCLIENTE    = $params[2]['value'];
            $CDCONSUMIDOR = $params[3]['value'];
            $CDFAMILISALD = $params[4]['value'];
            $CDPRODUTO    = $params[5]['value'];
            $cdWeekDays   = $params[6]['value'];

            if(count($cdWeekDays) === 7){
                $this->restrictionService->updateProductRestriction($QTCONSDIARAL, $CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO, 'SSSSSSS');
            } else{
                $IDBLOQDIARAL = '';

                // Domingo
                if(in_array('1', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Segunda
                if(in_array('2', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Terça
                if(in_array('3', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Quarta
                if(in_array('4', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Quinta
                if(in_array('5', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Sexta
                if(in_array('6', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                // Sabado
                if(in_array('7', $cdWeekDays)){
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'S';
                } else{
                    $IDBLOQDIARAL = $IDBLOQDIARAL.'N';
                }

                $this->restrictionService->updateProductRestriction($QTCONSDIARAL, $CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO, $IDBLOQDIARAL);
            }

            $restrictions = $this->restrictionService->getProductRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);

            foreach ($restrictions as &$prodRestriction){
                $prodRestriction['cdWeekDays'] = array();
                if($prodRestriction['IDBLOQDIARAL'] !== null){
                    for($i = 1; $i <= 7; $i++){
                        if($prodRestriction['IDBLOQDIARAL'][$i-1] === 'S'){
                            array_push($prodRestriction['cdWeekDays'], "$i");
                        }
                    }
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductRestrictionsRepository', $restrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* Deletes a product restriction. */
    public function deleteProductRestriction(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL     = $params[0]['value'];
            $CDCLIENTE    = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];
            $CDFAMILISALD = $params[3]['value'];
            $CDPRODUTO    = $params[4]['value'];

            $this->restrictionService->deleteProductRestriction($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $CDPRODUTO);

            $restrictions = $this->restrictionService->getProductRestrictions($CDFILIAL, $CDCLIENTE, $CDCONSUMIDOR);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ProductRestrictionsRepository', $restrictions));

        } catch (\Exception $e){
            $response->addMessage(new Message($e->getMessage()));
        }
    }

}