describe('MessageService', function() {

	var zhPromise = new ZHPromise(Q);
	var messageService;
	var messages = {
		'default': {
			'CODE1': 'This is the test number: __number__'
		},
		'custom_package': {
			'CODE2': 'This is a custom package message'
		}
	};

	var timeout = function(callback) {
		callback();
	};

	var templateManager = {
		ZEEDHI_PACKAGE: 'default'
	};

	var screenServiceMock;

	var injector = {
		get: function () {
			return {
				loadMessagesFile: function(pckg) {
					if (messages[pckg] === undefined) {
						return zhPromise.reject('File not found');
					} else {
						return zhPromise.when(messages[pckg]);
					}
				}
			};
		}
	};

	var zhLogger = {
		error: function() {}
	};

	beforeEach(function() {
		screenServiceMock = jasmine.createSpyObj([
			'showMessage',
			'confirmMessage',
			'showCustomDialog',
			'notificationMessage',
			'alertNotification',
			'successNotification',
			'errorNotification'
		]);

		messageService = new MessageService(screenServiceMock, templateManager, injector, timeout);
	});

	it('showMessageByCode should get message from default package', function(done) {
		messageService.getMessageByCode('CODE1').then(function(message) {
			expect(message).toBe('This is the test number: __number__');
			done();
		});
	});

	it('showMessageByCode should get message from custom package', function(done) {
		messageService.getMessageByCode('custom_package#CODE2').then(function(message) {
			expect(message).toBe('This is a custom package message');
			done();
		});
	});

	it('showMessageByCode should reject when package does not exists', function(done) {
		messageService.getMessageByCode('not_existed_package#MESSAGE').then(function(message) {
			fail('The promise should have been rejected');
		}).catch(function(error) {
			expect(error).toBe('File not found');
			done();
		});
	});

	it('showMessageByCode should reject when it can\'t find the message', function(done) {
		messageService.getMessageByCode('CODE0').then(function() {
			fail('The promise should have been rejected');
		}).catch(function() {
			expect(screenServiceMock.showMessage).not.toHaveBeenCalled();
			done();
		});
	});

	it('showMessageByCode should get the correct message', function(done) {
		messageService.showMessageByCode('CODE1', {
			number: 10
		}).then(function() {
			var message = screenServiceMock.showMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.showMessage.calls.mostRecent().args[3].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(10);
			done();
		});
	});

	it('confirmMessageByCode should get the correct message', function(done) {
		messageService.confirmMessageByCode('CODE1', {
			number: 20
		}).then(function() {
			var message = screenServiceMock.confirmMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.confirmMessage.calls.mostRecent().args[5].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(20);
			done();
		});
	});

	it('showCustomDialogByCode should get the correct message', function(done) {
		var message;

		messageService.showCustomDialogByCode('CODE1', {
			number: 30
		}).then(function() {
			var message = screenServiceMock.showCustomDialog.calls.mostRecent().args[0];
			var number = screenServiceMock.showCustomDialog.calls.mostRecent().args[4].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(30);
			done();
		});
	});

	it('notificationMessageByCode should get the correct message', function(done) {
		messageService.notificationMessageByCode('CODE1', {
			number: 40
		}, {}).then(function() {
			var message = screenServiceMock.notificationMessage.calls.mostRecent().args[0];
			var number = screenServiceMock.notificationMessage.calls.mostRecent().args[3].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(40);
			done();
		});
	});

	it('alertNotificationByCode should get the correct message', function(done) {
		messageService.alertNotificationByCode('CODE1', 0, {
			number: 50
		}).then(function() {
			var message = screenServiceMock.alertNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.alertNotification.calls.mostRecent().args[2].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(50);
			done();
		});
	});

	it('successNotificationByCode should get the correct message', function(done) {
		messageService.successNotificationByCode('CODE1', 0, {
			number: 60
		}).then(function() {
			var message = screenServiceMock.successNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.successNotification.calls.mostRecent().args[2].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(60);
			done();
		});
	});

	it('errorNotificationByCode should get the correct message', function(done) {
		messageService.errorNotificationByCode('CODE1', {
			number: 70
		}).then(function() {
			var message = screenServiceMock.errorNotification.calls.mostRecent().args[0];
			var number = screenServiceMock.errorNotification.calls.mostRecent().args[1].number;
			expect(message).toEqual('This is the test number: __number__');
			expect(number).toEqual(70);
			done();
		});
	});
});